/*  Projecte de monitorització del so ambiental d’un determinat espai.
    Utilitzem un semàfor que ens indicarà un so baix (verd), un so mitjà (groc) i un so alt (vermell).
    El senyal elèctric que envia el micròfon de ganància ajustable GY-MAX4466 al pin analògic A0, 
    és major com major és la intensitat del so ambiental. 
    En la mesura de la intensitat del so, la tarja Arduino UNO pren per referència el valor de 3,3V que entra pel pin AREF.
    El valor puntual d'una medició de so varia molt ràpidament i també capta molt de soroll que cal descartar. 
    Es defineix un espai temporal de recollida de mostra de 50 ms, que equival a una freqüència de 20 Hz.
    Durant aquest espai de medició de 50 ms prenem mostres del senyal i seleccionem el màxim i el mínim registrat
    i a partir d'aquí en deduïm una "mostra" més estable.
    (Modificació del codi del sketch d'exemple de nivell de so del micròfon amplificat Adafruit)
*/

const int    tempsRecollidaMostra = 50; // ample de la finestra temporal de lectura del senyal en mS (50 mS = 20Hz)
unsigned int mostra;                    // recollida de lectures constants durant la finestra temporal (max - min)
int          ledVerd = 6; 
int          ledGroc = 5;
int          ledRoig = 4;

void setup() 
{
   pinMode(A0, INPUT);
   pinMode(ledVerd, OUTPUT);
   pinMode(ledGroc, OUTPUT);
   pinMode(ledRoig, OUTPUT);
   Serial.begin(9600);                  // inicialització del Monitor Sèrie
   analogReference(EXTERNAL);           // utilització del pin AREF com a refèrencia de voltatge (3,3V)
}

void loop() 
{
   unsigned long iniciRecollidaMostra= millis();     // punt d'inici de la finestra temporal de lectura del senyal en mS
   unsigned int picApic = 0;                       // valor del so picApic
   unsigned int senyalMax = 0;
   unsigned int senyalMin = 1024;
   while (millis() - iniciRecollidaMostra < tempsRecollidaMostra) // recull mostres durant 50 mS
   {
      mostra = analogRead(0);
      if (mostra < 1024)                             // descarta lectures erràtiques
      {
         if (mostra > senyalMax)
         {
            senyalMax = mostra;                      // guarda només el nivell màxim
         }
         else if (mostra < senyalMin)
         {
            senyalMin = mostra;                      // guarda només el nivell mínim
         }
      }
   }
   picApic = senyalMax - senyalMin;      // màxim - mínim = amplada picApic
   double volts = (picApic * 3.3) / 1023;// converteix en volts
   
   Serial.print("Mostra (0..1023): ");   // presenta els valors pel monitor sèrie
   Serial.print(mostra);
   Serial.print(" Volts (0..3,3): ");
   Serial.println(volts);
       
   if(volts < 0.29){                        // encén els leds segons els valors
    analogWrite(ledVerd, 255);
   }
   else{
    analogWrite(ledVerd, 0);
   }
   if(volts > 0.30 && volts < 0.49){
    analogWrite(ledGroc, 255);
   }
   else{
    analogWrite(ledGroc, 0);
   }
   if(volts > 0.50){
    analogWrite(ledRoig, 255);
   }
   else{
    analogWrite(ledRoig,0);
   }
}
